/**
 * @file Archivo sobre la pestaña 1, el line chart
 * @author David Población Criado
 */

const COLORES = { "Ávila": '#e41a1c', "Burgos": '#377eb8', "León": '#4daf4a', "Palencia": '#984ea3', "Salamanca": '#ff7f00', "Segovia": '#b8b800', "Soria": '#a65628', "Valladolid": '#f781bf', "Zamora": '#999999' };

/**
 * Selecciona los datos de unas provincias en concreto
 * @param {JSON} data Datos de los que se quiere obtener el subconjunto
 * @param {String[]} provincias Provincias a seleccionar de los datos
 */
function seleccionarDatosProvincias(data, provincias) {
    var dataSelec = [];
    data.forEach(function (d) {
        if (provincias.includes(d.Provincia)) {
            dataSelec.push(d);
        }
    })
    return dataSelec;
}
/**
 * Animaciones del linechart, para que se vayan mostrando las líneas
 * gradualmente de izquierda a derecha
 * @param {*} path 
 */
function transition(path) {
    path.transition()
        .duration(4000)
        .attrTween("stroke-dasharray", tweenDash)
        .on("end", () => { d3.select(this).call(transition); });
}

function tweenDash() {
    if (this instanceof Window)
        return;
    const l = this.getTotalLength(),
        i = d3.interpolateString("0," + l, l + "," + l);
    return function (t) { return i(t) };
}

/**
 * Dibuja el gráfico de la pestaña 1, correspondiente al line chart
 * @param {String[]} provincias Provincias de las cuales se quiere ver el gráfico
 * @param {int} tipoConsumo Para elegir uno u otro tipo de consumo:
 *      - 0: Consumo total
 *      - 1: Consumo per cápita
 */
function dibujarGrafico1(provincias, tipoConsumo) {
    d3.csv("./data/d1.csv").then(function (data) {
        var margin = { top: 30, right: 30, bottom: 70, left: 120 },
            width = window.innerWidth - 500 - margin.left - margin.right,
            height = 400 - margin.top - margin.bottom;
        var parseMes = d3.timeParse("%Y-%m");

        // Tratamiento de datos
        data.forEach(function (d) {
            d.Fecha = parseMes(d.Mes);
            if (tipoConsumo == 0)
                d.Consumo = +d.Consumo;
            else
                d.Consumo = +d.ConsumoHab;
        })
        var dataSel = seleccionarDatosProvincias(data, provincias);
        
        // Añadimos el svg al HTML
        var svg = d3.select("#serie")
            .append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .append("g")
            .attr("transform",
                "translate(" + margin.left + "," + margin.top + ")");

        // Agrupamos los datos, queremos una línea por provincia
        var agrupados = d3.nest() // usamos d3.nest, hay que importarlo aparte, otra librería
            .key(function (d) { return d.Provincia; })
            .entries(dataSel);

        // Añadimos el eje X el cual es un dato de fechas
        var x = d3.scaleTime()
            .domain(d3.extent(data, function (d) { return d.Fecha }))
            .rangeRound([0, width]);
        svg.append("g")
            .attr("transform", "translate(0," + height + ")")
            .call(d3.axisBottom(x).tickFormat(d3.timeFormat("%m-%Y")).ticks(16))
            // Giramos los labels del eje
            .selectAll("text")
            .style("text-anchor", "end")
            .attr("dx", "-.8em")
            .attr("dy", ".15em")
            .attr("transform", "rotate(-65)");

        // Añadimos el eje Y
        var y = d3.scaleLinear()
            .domain([0, d3.max(data, function (d) { return +d.Consumo; })])
            .range([height, 0]);
        svg.append("g")
            .call(d3.axisLeft(y));

        // Dibujamos las líneas
        svg.selectAll(".line")
            .data(agrupados)
            .enter()
            .append("path")
            .call(transition) // animación
            .attr("fill", "none")
            .attr("stroke", function (d) { return COLORES[d.key] }) // Elegimos los mismos colores
            .attr("stroke-width", 1.5)
            .attr("d", function (d) {
                return d3.line()
                    .x(function (d) { return x(d.Fecha); })
                    .y(function (d) { return y(d.Consumo); })
                    (d.values)
            });

        // Añadimos el grid del eje X
        svg.append("g")
            .attr("class", "grid")
            .attr("transform", "translate(0," + height + ")")
            .call(d3.axisBottom(x).ticks(16) // queremos todas las marcas
                .tickSize(-height)
                .tickFormat("")
            );

        // Añadimos el grid del eje Y
        svg.append("g")
            .attr("class", "grid")
            .call(d3.axisLeft(y)
                .tickSize(-width)
                .tickFormat("")
            );

        // Título del eje Y, diferente según tipoConsumo
        svg.append("text")
            .attr("transform", "rotate(-90)")
            .attr("y", tipoConsumo == 0 ? 10 - margin.left : 50 - margin.left)
            .attr("x", 0 - (height / 2))
            .attr("dy", "1em")
            .style("text-anchor", "middle")
            .text(tipoConsumo == 0 ? "Consumo (kWh)" : "Consumo per cápita (kWh/hab)");

        // Etiquetas de los puntos con el consumo en cada uno, las creamos
        // pero no las mostramos (solo se mostrarán cuando pasemos por encima)
        var tooltip = d3.select("body").append("div")
            .attr("class", "puntosSerie")
            .style("opacity", 0); // no se muestran
        
        var provincia = svg.selectAll(".provincia")
            .data(agrupados)
            .enter().append("g")
            .attr("class", "provincia");

        // Añadimos los puntos
        provincia.selectAll("circle")
            .data(function (d) { return d.values })
            .enter()
            .append("circle")
            .attr("r", 3)
            .attr("cx", function (d) { return x(d.Fecha); })
            .attr("cy", function (d) { return y(d.Consumo); })
            .style("fill", function (d) { return COLORES[d.Provincia]; })
            // cuando se pasa por encima, mostramos las etiquetas de los puntos y los hacemos grandes
            .on("mouseover", function (event) {
                d3.select(this).transition()
                    .duration('100')
                    .attr("r", 6);
                tooltip.transition()
                    .duration(100)
                    .style("opacity", 1);
                tooltip.html(d3.format(".2f")(this.__data__.Consumo) + (tipoConsumo == 0 ? " kWh" : " kWh/hab"))
                    .style("left", (event.pageX + 10))
                    .style("top", (event.pageY - 15))
                    .style("background", COLORES[this.__data__.Provincia]);

            })
            // cuando se quita el puntero, quitamos las etiquetas que sean visibles y restablecemos
            // el tamaño del punto
            .on("mouseout", function () {
                d3.select(this).transition()
                    .duration('200')
                    .attr("r", 3);
                // desaparecer etiquetas
                tooltip.transition()
                    .duration(100)
                    .style("opacity", 0)
            });
    })
}