/**
 * @file Archivo sobre la pestaña 2, el stacked area chart
 * @author David Población Criado
 */

/**
 * Dibuja el gráfico de la pestaña 2, correspondiente al stacked area chart
 * @param {String} provincia Provincia de las cual se quiere ver el gráfico, pueden ser todas
 *      las de CyL, incluido la totalidad de la región
 */
function dibujarGrafico2(provincia) {
    // cambiar titulo
    document.getElementById("titulo2").textContent = 'Consumo según suministrador en ' + provincia;

    d3.csv("./data/d2.csv").then(function(data) {
        var margin = { top: 30, right: 100, bottom: 70, left: 120 },
            width = 1000 - margin.left - margin.right,
            height = 400 - margin.top - margin.bottom;
        var parseMes = d3.timeParse("%Y-%m");

        // Tratamiento de fechas
        data.forEach(function(d) {
            d.Fecha = parseMes(d.Mes);
        })

        var dataSel = seleccionarDatosProvincias(data, provincia);
        var keys = data.columns.slice(2); // las claves serán los suministradores

        // Paleta de colores
        var color = d3.scaleOrdinal()
            .domain(keys)
            .range(d3.schemeSet2);

        // Apilamos los datos
        var stackedData = d3.stack()
            .keys(keys)
            (dataSel)
        // Añadimos el svg al HTML
        var svg = d3.select("#stacked")
            .append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .append("g")
            .attr("transform",
                "translate(" + margin.left + "," + margin.top + ")");

        // Añadimos el eje X
        var x = d3.scaleTime()
            .domain(d3.extent(data, function(d) { return d.Fecha }))
            .rangeRound([0, width]);
        var xAxis = svg.append("g")
            .attr("transform", "translate(0," + height + ")")
            .call(d3.axisBottom(x).tickFormat(d3.timeFormat("%m-%Y")).ticks(16))
            // Giramos los labels del eje
            .selectAll("text")
            .style("text-anchor", "end")
            .attr("dx", "-.8em")
            .attr("dy", ".15em")
            .attr("transform", "rotate(-65)");

        // Añadimos el eje Y, con 5 ticks (20,40,...,100)%
        var y = d3.scaleLinear()
            .domain([0, 100])
            .range([height, 0]);
        svg.append("g")
            .call(d3.axisLeft(y).ticks(5));

        // Título del eje Y
        svg.append("text")
            .attr("transform", "rotate(-90)")
            .attr("y", 40 - margin.left)
            .attr("x", 0 - (height / 2))
            .attr("dy", "1em")
            .style("text-anchor", "middle")
            .text("Consumo (%)");

        var clip = svg.append("defs").append("svg:clipPath")
            .attr("id", "clip")
            .append("svg:rect")
            .attr("width", width)
            .attr("height", height)
            .attr("x", 0)
            .attr("y", 0);

        // Brushing
        var brush = d3.brushX()
            .extent([
                [0, 0],
                [width, height]
            ]) // inicializa el area de brush, empieza en 0,0 y finaliza en width, height, selecciona toda la zona gráfica
            .on("end", updateChart)

        // Crea la variable scatter
        var areaChart = svg.append('g')
            .attr("clip-path", "url(#clip)")

        // Generador del area
        var area = d3.area()
            .x(function(d) { return x(d.data.Fecha); })
            .y0(function(d) { return y(d[0]); })
            .y1(function(d) { return y(d[1]); })

        // Mostramos el area
        areaChart
            .selectAll("mylayers")
            .data(stackedData)
            .enter()
            .append("path")
            .attr("class", function(d) { return "myArea " + d.key })
            .style("fill", function(d) { return color(d.key); })
            .attr("d", area);

        // Añadimos el brushing
        areaChart
            .append("g")
            .attr("class", "brush")
            .call(brush);

        var timeout;

        function idled() { timeout = null; }

        // Actualiza el gráfico
        function updateChart() {

            extent = d3.event.selection

            // Si no hay selección, volvemos al principio
            if (!extent) {
                if (!timeout) return timeout = setTimeout(idled, 350);
                x.domain(d3.extent(data, function(d) { return d.Fecha; }))
            } else {
                x.domain([x.invert(extent[0]), x.invert(extent[1])])
                areaChart.select(".brush").call(brush.move, null)
            }

            // Ajustamos la posición de los ejes y el área
            xAxis.transition().duration(1000).call(d3.axisBottom(x).ticks(5))
            areaChart
                .selectAll("path")
                .transition().duration(1000)
                .attr("d", area)
        }

        // Resaltado
        // Cuando estamos encima de un grupo, lo ponemos con color y a los demás se lo quitamos
        var resaltar = function(d) {
            // quitamos color de los demás
            d3.selectAll(".myArea").style("opacity", .1)
            // coloreamos
            d3.select("." + d.explicitOriginalTarget.data).style("opacity", 1)
        }

        // Cuando se termina la animación, el ratón está fuera
        var noResaltar = function(d) {
            d3.selectAll(".myArea").style("opacity", 1)
        }

        // Leyenda
        // Añadimos el cuadrado
        var size = 20
        svg.selectAll("myrect")
            .data(keys)
            .enter()
            .append("rect")
            .attr("x", 800)
            .attr("y", function(d, i) { return 10 + i * (size + 5) }) // 100 es donde aparece el primero. 25 distancia entre ellos
            .attr("width", size)
            .attr("height", size)
            .style("fill", function(d) { return color(d) })
            .on("mouseover", resaltar)
            .on("mouseleave", noResaltar)

        // Añadimos el texto
        svg.selectAll("mylabels")
            .data(keys)
            .enter()
            .append("text")
            .attr("x", 800 + size * 1.2)
            .attr("y", function(d, i) { return 10 + i * (size + 5) + (size / 2) }) // 100 es donde aparece el primero. 25 distancia entre ellos
            .style("fill", function(d) { return color(d) })
            .text(function(d) { return d })
            .attr("text-anchor", "left")
            .style("alignment-baseline", "middle")
            .on("mouseover", resaltar)
            .on("mouseleave", noResaltar)

    });
}