/**
 * @file Archivo sobre la pestaña 3, el grouped bar chart
 * @author David Población Criado
 */

/**
 * Dibuja el gráfico de la pestaña 3, correspondiente al grouped bar chart
 * @param {String} provincia Provincia de las cual se quiere ver el gráfico, pueden ser todas
 *      las de CyL, incluido la totalidad de la región
 */
function dibujarGrafico3(provincia) {
    // cambiar titulo
    document.getElementById("titulo3").textContent = 'Consumo por tipo de actividad en ' + provincia;
    d3.csv("./data/d3.csv").then(function(data) {
        var margin = { top: 30, right: 100, bottom: 70, left: 120 },
        width = window.innerWidth - 200 - margin.left - margin.right,
        height = 500 - margin.top - margin.bottom;
        var parseMes = d3.timeParse("%Y-%m");
        // Tratamiento de datos
        data.forEach(function(d) {
            d.Fecha = parseMes(d.Mes);
        })

        var dataSel = seleccionarDatosProvincias(data, provincia);
        var keys = data.columns.slice(2); // las claves serán los tipos de consumo

        // Añadimos el svg al HTML
        var svg = d3.select("#barChart")
            .append("svg")
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .append("g")
            .attr("transform",
                "translate(" + margin.left + "," + margin.top + ")");

        // La escala entre los distintos grupos de barras
        var x0 = d3.scaleBand()
            .rangeRound([0, width])
            .paddingInner(0.1);

        // Escala entre cada barra dentro del grupo
        var x1 = d3.scaleBand()
            .padding(0.05);

        // Escala del consumo
        var y = d3.scaleLinear()
            .rangeRound([height, 0]);

        // Paleta de colores
        var z = d3.scaleOrdinal()
            .range(["#98abc5", "#6b486b", "#ff8c00"]);

        x0.domain(data.map(function(d) { return d.Mes; }));
        x1.domain(keys).rangeRound([0, x0.bandwidth()]);
        y.domain([0, d3.max(dataSel, function(d) {
            return Math.max(d.Residencial, d.Servicios, d.Industrial)
        })]).nice();

        // Añadimos el grid del eje Y solamente
        svg.append("g")
            .attr("class", "grid")
            .call(d3.axisLeft(y)
                .tickSize(-width)
                .tickFormat("")
            );

        // Dibujamos las barras
        svg.append("g")
            .selectAll("g")
            .data(dataSel)
            .enter().append("g")
            .attr("class", "bar")
            .attr("transform", function(d) { return "translate(" + x0(d.Mes) + ",0)"; })
            .selectAll("rect")
            .data(function(d) { return keys.map(function(key) { return { key: key, value: d[key] }; }); })
            .enter()
            .append("rect")
            .on('mouseover', mouseover)
            .on('mousemove', mousemove)
            .on('mouseout', mouseout)
            .attr("x", function(d) { return x1(d.key); })
            .attr("y", d => { return height; })
            // animacion
            .transition()
            .duration(2000)
            .delay(function(d, i) {
                return i * 150;
            })
            .attr("y", function(d) { return y(d.value); })
            .attr("width", x1.bandwidth())
            .attr("height", function(d) { return height - y(d.value); })
            .attr("fill", function(d) { return z(d.key); });

        // Añadimos las etiquetas del eje X, para cada grupo
        svg.append("g")
            .attr("class", "axis")
            .attr("transform", "translate(0," + height + ")")
            .call(d3.axisBottom(x0)).selectAll("text")
            .style("text-anchor", "end")
            .attr("dx", "-.8em")
            .attr("dy", ".15em")
            .attr("transform", "rotate(-65)");

        // Añadimos las etiquetas del eje Y
        svg.append("g")
            .attr("class", "y axis")
            .call(d3.axisLeft(y).ticks(10))
            .append("text")
            .attr("x", 2)
            .attr("y", y(y.ticks().pop()) + 0.5)
            .attr("dy", "0.32em")
            .attr("fill", "#000")
            .attr("font-weight", "bold")
            .attr("text-anchor", "start")
            .text("Consumo (kWh)");

        // Leyenda
        var leyenda = svg.append("g")
            .attr("font-family", "sans-serif")
            .attr("font-size", 10)
            .attr("text-anchor", "end")
            .selectAll("g")
            .data(keys.slice().reverse())
            .enter().append("g")
            .attr("transform", function(d, i) { return "translate(0," + i * 20 + ")"; });

        // Añadimos el cuadrado
        leyenda.append("rect")
            .attr("x", width - 17)
            .attr("width", 15)
            .attr("height", 15)
            .attr("fill", z)
            .attr("stroke", z)
            .attr("stroke-width", 2)
            .on("click", function(d) { update(d) }); // cuando se pulsa

        // Añadimos los tipos de consumo a la leyenda
        leyenda.append("text")
            .attr("x", width - 24)
            .attr("y", 9.5)
            .attr("dy", "0.32em")
            .text(function(d) { return d; });

        // Tooltips, cuando se pasa por encima el ratón para ver el consumo de cada barra
        var tooltip = d3.select('#barChart').append('div')
            .attr('class', 'tooltip')
            .style('display', 'none');

        // Si estamos encima, mostramos el consumo
        function mouseover(event) {
            tooltip.style('display', 'inline');
        }

        // Creamos el consumo y se va moviendo junto con el ratón
        function mousemove(event) {
            var d = d3.select(this).data()[0];
            tooltip
                .html(d.value + " kWh")
                .style('left', (event.pageX - 12) + 'px')
                .style('top', (event.pageY - 12) + 'px')
                .style('background', z(d.key));
        }

        // Se acaba la interacción cuando quitamos el ratón de la barra
        function mouseout() {
            tooltip.style('display', 'none');
        }

        // Interacción de quitar alguno de los tipos de consumo del gráfico
        var filtrados = [];
        function update(d) {
            // Añadimos a filtrados los datos que quitamos para no mostrarlos
            if (filtrados.indexOf(d.explicitOriginalTarget.__data__) == -1) {
                filtrados.push(d.explicitOriginalTarget.__data__);
                // Si hemos deseleccionado todos, volvemos a seleccionarlos todos, no tiene sentido no mostrar nada
                if (filtrados.length == keys.length) filtrados = [];
            } else {
                filtrados.splice(filtrados.indexOf(d), 1);
            }

            // Actualizamos las escalas para los datos filtrados
            var nuevasKeys = [];
            keys.forEach(function(d) {
                if (filtrados.indexOf(d) == -1) {
                    nuevasKeys.push(d);
                }
            })
            x1.domain(nuevasKeys).rangeRound([0, x0.bandwidth()]);
            y.domain([0, d3.max(dataSel, function(d) {
                // Calculamos el máximo dado que el eje y será el máximo de los que no tenemos en el filtro
                return Math.max(filtrados.includes("Residencial") ? null : d.Residencial, filtrados.includes("Servicios") ? null : d.Servicios, filtrados.includes("Industrial") ? null : d.Industrial)
            })]).nice();
            // eje y
            svg.select(".y")
                .transition()
                .call(d3.axisLeft(y).ticks(10))
                .duration(500);

            // Filtramos las barras que se tienen que quitar
            var barras = svg.selectAll(".bar").selectAll("rect")
                .data(function(d) { return keys.map(function(key) { return { key: key, value: d[key] }; }); })

            barras.filter(function(d) {
                    return filtrados.indexOf(d.key) > -1;
                })
                .transition()
                .attr("x", function(d) {
                    return (+d3.select(this).attr("x")) + (+d3.select(this).attr("width")) / 2;
                })
                .attr("height", 0)
                .attr("width", 0)
                .attr("y", function(d) { return height; })
                .duration(500);

            // Ajuste de las barras
            barras.filter(function(d) {
                    return filtrados.indexOf(d.key) == -1;
                })
                .transition()
                .attr("x", function(d) { return x1(d.key); })
                .attr("y", function(d) { return y(d.value); })
                .attr("height", function(d) { return height - y(d.value); })
                .attr("width", x1.bandwidth())
                .attr("fill", function(d) { return z(d.key); })
                .duration(500);


            // Actualizamos la leyenda
            leyenda.selectAll("rect")
                .transition()
                .attr("fill", function(d) {
                    if (filtrados.length) {
                        if (filtrados.indexOf(d) == -1) {
                            return z(d);
                        } else {
                            return "white";
                        }
                    } else {
                        return z(d);
                    }
                })
                .duration(100);
        }
    });
}